@@ -160,8 +160,10 @@ mod test {
         super::*,
         serde_json::Value,
         solana_sdk::{
+            bpf_loader_upgradeable,
             message::Message,
             pubkey::{self, Pubkey},
+            system_program, sysvar,
         },
     };
 
@@ -248,19 +250,16 @@ mod test {
     }
 
     #[test]
-    fn test_parse_bpf_upgradeable_loader_instructions() {
-        let mut keys: Vec<Pubkey> = vec![];
-        for _ in 0..8 {
-            keys.push(Pubkey::new_unique());
-        }
-        let offset = 4242;
-        let bytes = vec![8; 99];
+    fn test_parse_bpf_upgradeable_loader_create_buffer_ix() {
         let max_data_len = 54321;
 
-        let instructions = solana_sdk::bpf_loader_upgradeable::create_buffer(
-            &keys[0],
-            &keys[1],
-            &keys[2],
+        let payer_address = Pubkey::new_unique();
+        let buffer_address = Pubkey::new_unique();
+        let authority_address = Pubkey::new_unique();
+        let instructions = bpf_loader_upgradeable::create_buffer(
+            &payer_address,
+            &buffer_address,
+            &authority_address,
             55,
             max_data_len,
         )
@@ -269,30 +268,42 @@ mod test {
         assert_eq!(
             parse_bpf_upgradeable_loader(
                 &message.instructions[1],
-                &AccountKeys::new(&keys[0..3], None)
+                &AccountKeys::new(&message.account_keys, None)
             )
             .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "initializeBuffer".to_string(),
                 info: json!({
-                    "account": keys[1].to_string(),
-                    "authority": keys[2].to_string(),
+                    "account": buffer_address.to_string(),
+                    "authority": authority_address.to_string(),
                 }),
             }
         );
         assert!(parse_bpf_upgradeable_loader(
             &message.instructions[1],
-            &AccountKeys::new(&keys[0..2], None)
+            &AccountKeys::new(&message.account_keys[0..2], None)
         )
         .is_err());
+    }
 
-        let instruction =
-            solana_sdk::bpf_loader_upgradeable::write(&keys[1], &keys[0], offset, bytes.clone());
+    #[test]
+    fn test_parse_bpf_upgradeable_loader_write_ix() {
+        let offset = 4242;
+        let bytes = vec![8; 99];
+
+        let buffer_address = Pubkey::new_unique();
+        let authority_address = Pubkey::new_unique();
+        let instruction = bpf_loader_upgradeable::write(
+            &buffer_address,
+            &authority_address,
+            offset,
+            bytes.clone(),
+        );
         let message = Message::new(&[instruction], None);
         assert_eq!(
             parse_bpf_upgradeable_loader(
                 &message.instructions[0],
-                &AccountKeys::new(&keys[0..2], None)
+                &AccountKeys::new(&message.account_keys, None)
             )
             .unwrap(),
             ParsedInstructionEnum {
@@ -300,22 +311,36 @@ mod test {
                 info: json!({
                     "offset": offset,
                     "bytes": base64::encode(&bytes),
-                    "account": keys[1].to_string(),
-                    "authority": keys[0].to_string(),
+                    "account": buffer_address.to_string(),
+                    "authority": authority_address.to_string(),
                 }),
             }
         );
         assert!(parse_bpf_upgradeable_loader(
             &message.instructions[0],
-            &AccountKeys::new(&keys[0..1], None)
+            &AccountKeys::new(&message.account_keys[0..1], None)
         )
         .is_err());
+    }
 
-        let instructions = solana_sdk::bpf_loader_upgradeable::deploy_with_max_program_len(
-            &keys[0],
-            &keys[1],
-            &keys[4],
-            &keys[2],
+    #[test]
+    fn test_parse_bpf_upgradeable_loader_deploy_ix() {
+        let max_data_len = 54321;
+
+        let payer_address = Pubkey::new_unique();
+        let program_address = Pubkey::new_unique();
+        let buffer_address = Pubkey::new_unique();
+        let upgrade_authority_address = Pubkey::new_unique();
+        let programdata_address = Pubkey::find_program_address(
+            &[program_address.as_ref()],
+            &bpf_loader_upgradeable::id(),
+        )
+        .0;
+        let instructions = bpf_loader_upgradeable::deploy_with_max_program_len(
+            &payer_address,
+            &program_address,
+            &buffer_address,
+            &upgrade_authority_address,
             55,
             max_data_len,
         )
@@ -324,153 +349,198 @@ mod test {
         assert_eq!(
             parse_bpf_upgradeable_loader(
                 &message.instructions[1],
-                &AccountKeys::new(&keys[0..8], None)
+                &AccountKeys::new(&message.account_keys, None)
             )
             .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "deployWithMaxDataLen".to_string(),
                 info: json!({
                     "maxDataLen": max_data_len,
-                    "payerAccount": keys[0].to_string(),
-                    "programAccount": keys[1].to_string(),
-                    "authority": keys[2].to_string(),
-                    "programDataAccount": keys[3].to_string(),
-                    "bufferAccount": keys[4].to_string(),
-                    "rentSysvar": keys[5].to_string(),
-                    "clockSysvar": keys[6].to_string(),
-                    "systemProgram": keys[7].to_string(),
+                    "payerAccount": payer_address.to_string(),
+                    "programAccount": program_address.to_string(),
+                    "authority": upgrade_authority_address.to_string(),
+                    "programDataAccount": programdata_address.to_string(),
+                    "bufferAccount": buffer_address.to_string(),
+                    "rentSysvar": sysvar::rent::ID.to_string(),
+                    "clockSysvar": sysvar::clock::ID.to_string(),
+                    "systemProgram": system_program::ID.to_string(),
                 }),
             }
         );
         assert!(parse_bpf_upgradeable_loader(
             &message.instructions[1],
-            &AccountKeys::new(&keys[0..7], None)
+            &AccountKeys::new(&message.account_keys[0..7], None)
         )
         .is_err());
+    }
 
-        let instruction =
-            solana_sdk::bpf_loader_upgradeable::upgrade(&keys[2], &keys[3], &keys[0], &keys[4]);
+    #[test]
+    fn test_parse_bpf_upgradeable_loader_upgrade_ix() {
+        let program_address = Pubkey::new_unique();
+        let buffer_address = Pubkey::new_unique();
+        let authority_address = Pubkey::new_unique();
+        let spill_address = Pubkey::new_unique();
+        let programdata_address = Pubkey::find_program_address(
+            &[program_address.as_ref()],
+            &bpf_loader_upgradeable::id(),
+        )
+        .0;
+        let instruction = bpf_loader_upgradeable::upgrade(
+            &program_address,
+            &buffer_address,
+            &authority_address,
+            &spill_address,
+        );
         let message = Message::new(&[instruction], None);
         assert_eq!(
             parse_bpf_upgradeable_loader(
                 &message.instructions[0],
-                &AccountKeys::new(&keys[0..7], None)
+                &AccountKeys::new(&message.account_keys, None)
             )
             .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "upgrade".to_string(),
                 info: json!({
-                    "authority": keys[0].to_string(),
-                    "programDataAccount": keys[1].to_string(),
-                    "programAccount": keys[2].to_string(),
-                    "bufferAccount": keys[3].to_string(),
-                    "spillAccount": keys[4].to_string(),
-                    "rentSysvar": keys[5].to_string(),
-                    "clockSysvar": keys[6].to_string(),
+                    "authority": authority_address.to_string(),
+                    "programDataAccount": programdata_address.to_string(),
+                    "programAccount": program_address.to_string(),
+                    "bufferAccount": buffer_address.to_string(),
+                    "spillAccount": spill_address.to_string(),
+                    "rentSysvar": sysvar::rent::ID.to_string(),
+                    "clockSysvar": sysvar::clock::ID.to_string(),
                 }),
             }
         );
         assert!(parse_bpf_upgradeable_loader(
             &message.instructions[0],
-            &AccountKeys::new(&keys[0..6], None)
+            &AccountKeys::new(&message.account_keys[0..6], None)
         )
         .is_err());
+    }
 
-        let instruction =
-            solana_sdk::bpf_loader_upgradeable::set_buffer_authority(&keys[1], &keys[0], &keys[2]);
+    #[test]
+    fn test_parse_bpf_upgradeable_loader_set_buffer_authority_ix() {
+        let buffer_address = Pubkey::new_unique();
+        let current_authority_address = Pubkey::new_unique();
+        let new_authority_address = Pubkey::new_unique();
+        let instruction = bpf_loader_upgradeable::set_buffer_authority(
+            &buffer_address,
+            &current_authority_address,
+            &new_authority_address,
+        );
         let message = Message::new(&[instruction], None);
         assert_eq!(
             parse_bpf_upgradeable_loader(
                 &message.instructions[0],
-                &AccountKeys::new(&keys[0..3], None)
+                &AccountKeys::new(&message.account_keys, None)
             )
             .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "setAuthority".to_string(),
                 info: json!({
-                    "account": keys[1].to_string(),
-                    "authority": keys[0].to_string(),
-                    "newAuthority": keys[2].to_string(),
+                    "account": buffer_address.to_string(),
+                    "authority": current_authority_address.to_string(),
+                    "newAuthority": new_authority_address.to_string(),
                 }),
             }
         );
         assert!(parse_bpf_upgradeable_loader(
             &message.instructions[0],
-            &AccountKeys::new(&keys[0..1], None)
+            &AccountKeys::new(&message.account_keys[0..1], None)
         )
         .is_err());
+    }
 
-        let instruction = solana_sdk::bpf_loader_upgradeable::set_upgrade_authority(
-            &keys[1],
-            &keys[0],
-            Some(&keys[2]),
+    #[test]
+    fn test_parse_bpf_upgradeable_loader_set_upgrade_authority_ix() {
+        let program_address = Pubkey::new_unique();
+        let current_authority_address = Pubkey::new_unique();
+        let new_authority_address = Pubkey::new_unique();
+        let (programdata_address, _) = Pubkey::find_program_address(
+            &[program_address.as_ref()],
+            &bpf_loader_upgradeable::id(),
+        );
+        let instruction = bpf_loader_upgradeable::set_upgrade_authority(
+            &program_address,
+            &current_authority_address,
+            Some(&new_authority_address),
         );
         let message = Message::new(&[instruction], None);
         assert_eq!(
             parse_bpf_upgradeable_loader(
                 &message.instructions[0],
-                &AccountKeys::new(&keys[0..3], None)
+                &AccountKeys::new(&message.account_keys, None)
             )
             .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "setAuthority".to_string(),
                 info: json!({
-                    "account": keys[1].to_string(),
-                    "authority": keys[0].to_string(),
-                    "newAuthority": keys[2].to_string(),
+                    "account": programdata_address.to_string(),
+                    "authority": current_authority_address.to_string(),
+                    "newAuthority": new_authority_address.to_string(),
                 }),
             }
         );
         assert!(parse_bpf_upgradeable_loader(
             &message.instructions[0],
-            &AccountKeys::new(&keys[0..1], None)
+            &AccountKeys::new(&message.account_keys[0..1], None)
         )
         .is_err());
 
-        let instruction =
-            solana_sdk::bpf_loader_upgradeable::set_upgrade_authority(&keys[1], &keys[0], None);
+        let instruction = bpf_loader_upgradeable::set_upgrade_authority(
+            &program_address,
+            &current_authority_address,
+            None,
+        );
         let message = Message::new(&[instruction], None);
         assert_eq!(
             parse_bpf_upgradeable_loader(
                 &message.instructions[0],
-                &AccountKeys::new(&keys[0..2], None)
+                &AccountKeys::new(&message.account_keys, None)
             )
             .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "setAuthority".to_string(),
                 info: json!({
-                    "account": keys[1].to_string(),
-                    "authority": keys[0].to_string(),
+                    "account": programdata_address.to_string(),
+                    "authority": current_authority_address.to_string(),
                     "newAuthority": Value::Null,
                 }),
             }
         );
         assert!(parse_bpf_upgradeable_loader(
             &message.instructions[0],
-            &AccountKeys::new(&keys[0..1], None)
+            &AccountKeys::new(&message.account_keys[0..1], None)
         )
         .is_err());
+    }
 
-        let instruction = solana_sdk::bpf_loader_upgradeable::close(&keys[0], &keys[1], &keys[2]);
+    #[test]
+    fn test_parse_bpf_upgradeable_loader_close_ix() {
+        let close_address = Pubkey::new_unique();
+        let recipient_address = Pubkey::new_unique();
+        let authority_address = Pubkey::new_unique();
+        let instruction =
+            bpf_loader_upgradeable::close(&close_address, &recipient_address, &authority_address);
         let message = Message::new(&[instruction], None);
         assert_eq!(
             parse_bpf_upgradeable_loader(
                 &message.instructions[0],
-                &AccountKeys::new(&keys[..3], None)
+                &AccountKeys::new(&message.account_keys, None)
             )
             .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "close".to_string(),
                 info: json!({
-                    "account": keys[1].to_string(),
-                    "recipient": keys[2].to_string(),
-                    "authority": keys[0].to_string(),
+                    "account": close_address.to_string(),
+                    "recipient": recipient_address.to_string(),
+                    "authority": authority_address.to_string(),
                 }),
             }
         );
         assert!(parse_bpf_upgradeable_loader(
             &message.instructions[0],
-            &AccountKeys::new(&keys[0..1], None)
+            &AccountKeys::new(&message.account_keys[0..1], None)
         )
         .is_err());
     }
